@@ -26,7 +26,13 @@
 #include <QToolTip>
 
 #if HAVE_X11
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
 #include <QX11Info>
+#endif
+
 #include <netwm.h>
 #endif
 
@@ -67,7 +73,7 @@ public:
             q->setPalette(QToolTip::palette());
         }
         connect(hideTimer, &QTimer::timeout, q, &QWidget::hide);
-        connect(q, QOverload<>::of(&KPassivePopup::clicked), q, &QWidget::hide);
+        connect(q, qOverload<>(&KPassivePopup::clicked), q, &QWidget::hide);
     }
 
     KPassivePopup *const q;
@@ -101,7 +107,8 @@ public:
         const int width = q->width();
         const int height = q->height();
 
-        int xh = 70, xl = 40;
+        int xh = 70;
+        int xl = 40;
         if (width < 80) {
             xh = xl = 40;
         } else if (width < 110) {
@@ -124,7 +131,8 @@ public:
         QBrush brush(Qt::color1, Qt::SolidPattern);
         p.setBrush(brush);
 
-        int i = 0, z = 0;
+        int i = 0;
+        int z = 0;
         for (; i < 4; ++i) {
             QPainterPath path;
             path.moveTo(corners[i].x(), corners[i].y());
@@ -249,10 +257,7 @@ KPassivePopup::KPassivePopup(WId win)
 {
 }
 
-KPassivePopup::~KPassivePopup()
-{
-    delete d;
-}
+KPassivePopup::~KPassivePopup() = default;
 
 void KPassivePopup::setPopupStyle(int popupstyle)
 {
